@@ -1,5 +1,5 @@
 /*
-    $Id: addressobj.c 3086 2023-09-03 06:23:08Z soci $
+    $Id: addressobj.c 3211 2025-04-12 18:45:16Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 #include "eval.h"
 #include "variables.h"
 #include "arguments.h"
-#include "instruction.h"
 
 #include "boolobj.h"
 #include "strobj.h"
@@ -34,6 +33,7 @@
 #include "bitsobj.h"
 #include "bytesobj.h"
 #include "registerobj.h"
+#include "codeobj.h"
 
 static Type obj;
 
@@ -41,6 +41,12 @@ Type *const ADDRESS_OBJ = &obj;
 
 static MUST_CHECK Obj *address_from_obj(Obj *v1, linepos_t epoint) {
     switch (v1->obj->type) {
+    case T_CODE:
+        if (Code(v1)->typ->obj == ADDRESS_OBJ) {
+            atype_t am = Address(Code(v1)->typ)->type;
+            return new_address(code_remove_address(Code(v1), true), am);
+        }
+        FALL_THROUGH; /* fall through */
     case T_BOOL:
     case T_INT:
     case T_BITS:
@@ -244,8 +250,9 @@ static inline bool check_addr2(atype_t type) {
     return false;
 }
 
-Address_types register_to_indexing(unsigned int c) {
-    switch (c) {
+Address_types register_to_indexing(const Register *reg) {
+    if (reg->len != 1) return A_NONE;
+    switch (reg->data[0]) {
     case 's': return A_SR;
     case 'r': return A_RR;
     case 'z': return A_ZR;
@@ -474,16 +481,6 @@ static MUST_CHECK Obj *calc2(oper_t op) {
                     if (am1 == A_NONE) return result;
                     return new_address(result, am1);
                 }
-            case O_XOR:
-                if (check_addr(am)) break;
-                if (check_addr(am2)) break;
-                if (am == am2) {
-                    op->v1 = v1->val;
-                    op->v2 = v2->val;
-                    op->inplace = NULL;
-                    return op->v1->obj->calc2(op);
-                }
-                break;
             default:
                 break;
             }
@@ -506,7 +503,7 @@ static MUST_CHECK Obj *calc2(oper_t op) {
         case O_MAX:
         case O_GT:
         case O_GE:
-            if (am == A_NONE || (am == A_DR && dpage == 0) || (am == A_BR && databank == 0)) {
+            if (am == A_NONE) {
                 op->v1 = v1->val;
                 op->inplace = NULL;
                 return op->v1->obj->calc2(op);
@@ -527,11 +524,13 @@ static MUST_CHECK Obj *calc2(oper_t op) {
             op->v1 = v1->val;
             op->inplace = NULL;
             return new_address(op->v1->obj->calc2(op), am);
+        case O_MEMBER:
+        case O_X: break;
         }
         break;
     case T_REGISTER:
-        if (Register(op->v2)->len == 1) {
-            Address_types am2 = register_to_indexing(Register(op->v2)->data[0]);
+        {
+            Address_types am2 = register_to_indexing(Register(op->v2));
             if (am2 == A_NONE) break;
             if (op->op == O_ADD) {
                 return new_address(val_reference(v1->val), v1->type << 4 | am2);
@@ -570,6 +569,7 @@ static MUST_CHECK Obj *calc2(oper_t op) {
                 }
                 if (am2 != A_NONE) break;
                 if (am1 != A_NONE) while ((am1 & 0xf) == A_NONE) am1 >>= 4;
+                if (am1 == A_NONE) return val_reference(v1->val);
                 return new_address(val_reference(v1->val), am1);
             }
         }
@@ -621,19 +621,17 @@ static MUST_CHECK Obj *rcalc2(oper_t op) {
     case T_CODE:
         return t1->calc2(op);
     case T_REGISTER:
-        if (Register(op->v1)->len == 1) {
-            am = register_to_indexing(Register(op->v1)->data[0]);
-            if (am == A_NONE) break;
-            if (op->op == O_ADD) {
-                return new_address(val_reference(v2->val), v2->type << 4 | am);
-            }
-            if (op->op == O_SUB) {
-                if (am == v2->type) {
-                    op->v1 = int_value[0];
-                    op->v2 = v2->val;
-                    op->inplace = NULL;
-                    return INT_OBJ->calc2(op);
-                }
+        am = register_to_indexing(Register(op->v1));
+        if (am == A_NONE) break;
+        if (op->op == O_ADD) {
+            return new_address(val_reference(v2->val), v2->type << 4 | am);
+        }
+        if (op->op == O_SUB) {
+            if (am == v2->type) {
+                op->v1 = int_value[0];
+                op->v2 = v2->val;
+                op->inplace = NULL;
+                return INT_OBJ->calc2(op);
             }
         }
         break;
