@@ -1,5 +1,5 @@
 /*
-    $Id: arguments.c 3115 2023-09-07 06:06:04Z soci $
+    $Id: arguments.c 3225 2025-04-24 05:56:46Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -369,7 +369,7 @@ enum {
     OUTPUT_APPEND, NO_OUTPUT, ERROR_APPEND, NO_ERROR, LABELS_APPEND, MAP,
     NO_MAP, MAP_APPEND, LIST_APPEND, SIMPLE_LABELS, LABELS_SECTION,
     MESEN_LABELS, LABELS_ADD_PREFIX, MAKE_APPEND, C256_PGX, C256_PGZ,
-    OUTPUT_EXEC
+    OUTPUT_EXEC, M45GS02, CTAGS_LABELS, CODY_BIN, WDC_BIN
 };
 
 static const struct my_option long_options[] = {
@@ -390,6 +390,8 @@ static const struct my_option long_options[] = {
     {"cbm-prg"          , my_no_argument      , NULL,  CBM_PRG},
     {"c256-pgx"         , my_no_argument      , NULL,  C256_PGX},
     {"c256-pgz"         , my_no_argument      , NULL,  C256_PGZ},
+    {"cody-bin"         , my_no_argument      , NULL,  CODY_BIN},
+    {"wdc-bin"          , my_no_argument      , NULL,  WDC_BIN},
     {"no-ascii"         , my_no_argument      , NULL,  NO_ASCII},
     {"ascii"            , my_no_argument      , NULL, 'a'},
     {"no-tasm-compatible",my_no_argument      , NULL,  NO_TASM_COMPATIBLE},
@@ -408,6 +410,7 @@ static const struct my_option long_options[] = {
     {"mr65c02"          , my_no_argument      , NULL,  MR65C02},
     {"mw65c02"          , my_no_argument      , NULL,  MW65C02},
     {"m4510"            , my_no_argument      , NULL,  M4510},
+    {"m45gs02"          , my_no_argument      , NULL,  M45GS02},
     {"labels"           , my_required_argument, NULL, 'l'},
     {"labels-append"    , my_required_argument, NULL,  LABELS_APPEND},
     {"output"           , my_required_argument, NULL, 'o'},
@@ -428,6 +431,7 @@ static const struct my_option long_options[] = {
     {"dump-labels"      , my_no_argument      , NULL,  DUMP_LABELS},
     {"simple-labels"    , my_no_argument      , NULL,  SIMPLE_LABELS},
     {"mesen-labels"     , my_no_argument      , NULL,  MESEN_LABELS},
+    {"ctags-labels"     , my_no_argument      , NULL,  CTAGS_LABELS},
     {"labels-add-prefix", my_required_argument, NULL,  LABELS_ADD_PREFIX},
     {"labels-root"      , my_required_argument, NULL,  LABELS_ROOT},
     {"labels-section"   , my_required_argument, NULL,  LABELS_SECTION},
@@ -506,8 +510,10 @@ static address_t check_outputs(void) {
         case OUTPUT_IHEX:
         case OUTPUT_SREC:
         case OUTPUT_FLAT: min &= 0xffffffff; break;
+        case OUTPUT_WDC:
         case OUTPUT_PGX:
         case OUTPUT_PGZ: min &= 0xffffff; break;
+        case OUTPUT_CODY:
         case OUTPUT_MHEX:
         case OUTPUT_APPLE:
         case OUTPUT_XEX: min &= 0xffff; break;
@@ -633,6 +639,8 @@ int init_arguments(int *argc2, char **argv2[]) {
             case CBM_PRG: output.mode = OUTPUT_CBM;break;
             case C256_PGX: output.mode = OUTPUT_PGX;break;
             case C256_PGZ: output.mode = OUTPUT_PGZ;break;
+            case CODY_BIN: output.mode = OUTPUT_CODY;break;
+            case WDC_BIN: output.mode = OUTPUT_WDC;break;
             case 'b':output.mode = OUTPUT_RAW;break;
             case 'f':output.mode = OUTPUT_FLAT;break;
             case 'a':arguments.to_ascii = true;break;
@@ -687,6 +695,7 @@ int init_arguments(int *argc2, char **argv2[]) {
             case MR65C02: arguments.cpumode = &r65c02;break;
             case MW65C02: arguments.cpumode = &w65c02;break;
             case M4510: arguments.cpumode = &c4510;break;
+            case M45GS02: arguments.cpumode = &c45gs02;break;
             case LABELS_APPEND:
             case 'l': symbol_output.name = my_optarg;
                       symbol_output.append = (opt == LABELS_APPEND);
@@ -706,6 +715,7 @@ int init_arguments(int *argc2, char **argv2[]) {
             case DUMP_LABELS: symbol_output.mode = LABEL_DUMP; break;
             case SIMPLE_LABELS: symbol_output.mode = LABEL_SIMPLE; break;
             case MESEN_LABELS: symbol_output.mode = LABEL_MESEN; break;
+            case CTAGS_LABELS: symbol_output.mode = LABEL_CTAGS; break;
             case LABELS_ROOT: get_arg(&get_args, &symbol_output.space_pos); break;
             case LABELS_SECTION: symbol_output.section = my_optarg; break;
             case LABELS_ADD_PREFIX: symbol_output.add_prefix = my_optarg; break;
@@ -742,15 +752,16 @@ int init_arguments(int *argc2, char **argv2[]) {
                "        [-E <file>] [-I <path>] [-l <file>] [-L <file>] [-M <file>] [--ascii]\n"
                "        [--nostart] [--long-branch] [--case-sensitive] [--cbm-prg] [--flat]\n"
                "        [--atari-xex] [--apple-ii] [--intel-hex] [--mos-hex] [--s-record]\n"
-               "        [--nonlinear] [--c256-pgx] [--c256-pgz] [--tasm-compatible]\n"
-               "        [--long-address] [--output-section=<name>] [--m65c02] [--m6502]\n"
-               "        [--m65xx] [--m65dtv02] [--m65816] [--m65el02] [--mr65c02] [--mw65c02]\n"
-               "        [--m65ce02] [--m4510] [--labels=<file>] [--labels-append=<file>]\n"
-               "        [--labels-add-prefix=<txt>] [--labels-section=<name>]\n"
-               "        [--labels-root=<expr>] [--export-labels] [--vice-labels-numeric]\n"
-               "        [--vice-labels] [--dump-labels] [--simple-labels] [--mesen-labels]\n"
-               "        [--list=<file>] [--list-append=<file>] [--no-monitor] [--no-source]\n"
-               "        [--line-numbers] [--tab-size=<value>] [--verbose-list] [-W<option>]\n"
+               "        [--nonlinear] [--c256-pgx] [--c256-pgz] [--cody-bin] [--wdc-bin]\n"
+               "        [--tasm-compatible] [--long-address] [--output-section=<name>]\n"
+               "        [--m65c02] [--m6502] [--m65xx] [--m65dtv02] [--m65816] [--m65el02]\n"
+               "        [--mr65c02] [--mw65c02] [--m65ce02] [--m4510] [--m45gs02]\n"
+               "        [--labels=<file>] [--labels-append=<file>] [--labels-add-prefix=<txt>]\n"
+               "        [--labels-section=<name>] [--labels-root=<expr>] [--export-labels]\n"
+               "        [--vice-labels-numeric] [--vice-labels] [--dump-labels]\n"
+               "        [--simple-labels] [--mesen-labels] [--ctags-labels] [--list=<file>]\n"
+               "        [--list-append=<file>] [--no-monitor] [--no-source] [--line-numbers]\n"
+               "        [--tab-size=<value>] [--verbose-list] [-W<option>]\n"
                "        [--dependencies=<file>] [--dependencies-append=<file>] [--make-phony]\n"
                "        [--output=<file>] [--output-append=<file>] [--output-exec=<expr>]\n"
                "        [--no-output] [--map=<file>] [--map-append=<file>] [--no-map]\n"
@@ -846,6 +857,8 @@ int init_arguments(int *argc2, char **argv2[]) {
                "      --s-record         Output Motorola S-record file\n"
                "      --c256-pgx         Output C256 PGX file\n"
                "      --c256-pgz         Output C256 PGZ file\n"
+               "      --cody-bin         Output Cody binary file\n"
+               "      --wdc-bin          Output WDC binary file\n"
                "\n"
                " Target CPU selection:\n"
                "      --m65xx            Standard 65xx (default)\n"
@@ -858,6 +871,7 @@ int init_arguments(int *argc2, char **argv2[]) {
                "      --mr65c02          R65C02\n"
                "      --mw65c02          W65C02\n"
                "      --m4510            CSG 4510\n"
+               "      --m45gs02          45GS02\n"
                "\n"
                " Source listing and labels:\n"
                "  -l, --labels=<file>    List labels into <file>\n"
@@ -868,6 +882,7 @@ int init_arguments(int *argc2, char **argv2[]) {
                "      --vice-labels-numeric Labels for VICE with numeric constants\n"
                "      --dump-labels      Dump for debugging\n"
                "      --simple-labels    Simple hexadecimal labels\n"
+               "      --ctags-labels     Tags file\n"
                "      --labels-root=<l>  List from scope <l> only\n"
                "      --labels-section=<n> List from section <n> only\n"
                "      --labels-add-prefix=<p> Set label prefix\n"
