@@ -1,5 +1,5 @@
 /*
-    $Id: error.h 3112 2023-09-06 06:34:22Z soci $
+    $Id: error.h 3206 2025-04-12 08:52:34Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -52,7 +52,8 @@ extern void err_msg(Error_types, const void *);
 extern void err_msg2(Error_types, const void *, linepos_t);
 extern void err_msg_wrong_type2(const struct Obj *, struct Type *, linepos_t);
 extern void err_msg_invalid_namespace_conv(const struct values_s *);
-extern void err_msg_cant_unpack(size_t, size_t, linepos_t);
+extern void err_msg_cant_unpack(argcount_t, size_t, struct Obj *, linepos_t);
+extern void err_msg_for_no_value(struct Obj *, linepos_t);
 extern void err_msg_cant_calculate(const struct str_t *, linepos_t);
 extern void err_msg_cant_calculate2(const struct str_t *, const struct file_list_s *, linepos_t);
 extern void err_msg_still_none(const struct str_t *, linepos_t);
@@ -85,7 +86,7 @@ extern void err_msg_bool(Error_types, struct Obj *, linepos_t);
 extern void err_msg_bool_oper(struct oper_s *);
 extern void err_msg_implied_reg(linepos_t, uint32_t);
 extern void err_msg_size_larger(linepos_t);
-extern void err_msg_jmp_bug(linepos_t);
+extern void err_msg_jmp_bug(struct Obj *, linepos_t);
 extern void err_msg_pc_bank(linepos_t);
 extern void err_msg_mem_wrap(linepos_t);
 extern void err_msg_addr_wrap(linepos_t);
@@ -106,6 +107,7 @@ extern void err_msg_wrong_character(linepos_t);
 extern void err_msg_encode_definition_note(const struct file_list_s *, linepos_t);
 extern void err_msg_star_assign(linepos_t);
 extern void err_msg_compound_note(linepos_t);
+extern void err_msg_string_expected(linepos_t);
 extern void err_msg_byte_note(linepos_t);
 extern void err_msg_char_note(const char *, linepos_t);
 extern void err_msg_immediate_note(linepos_t);
