@@ -1,5 +1,5 @@
 /*
-    $Id: errorobj.c 3068 2023-08-28 06:18:09Z soci $
+    $Id: errorobj.c 3227 2025-05-01 19:19:17Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -65,6 +65,7 @@ static FAST_CALL void destroy(Obj *o1) {
     case ERROR______NOT_ITER:
     case ERROR___MATH_DOMAIN:
     case ERROR_LOG_NON_POSIT:
+    case ERROR______LOG_BASE:
     case ERROR_SQUARE_ROOT_N:
     case ERROR___INDEX_RANGE:
     case ERROR_____KEY_ERROR:
@@ -72,6 +73,7 @@ static FAST_CALL void destroy(Obj *o1) {
     case ERROR_ZERO_NEGPOWER:
     case ERROR__NOT_ONE_CHAR:
     case ERROR______NOT_UVAL:
+    case ERROR_CANT_CROSS_BA:
         val_destroy(v1->u.obj);
         return;
     case ERROR__INVALID_CONV:
@@ -129,6 +131,7 @@ static FAST_CALL void garbage(Obj *o1, int i) {
     case ERROR______NOT_ITER:
     case ERROR___MATH_DOMAIN:
     case ERROR_LOG_NON_POSIT:
+    case ERROR______LOG_BASE:
     case ERROR_SQUARE_ROOT_N:
     case ERROR___INDEX_RANGE:
     case ERROR_____KEY_ERROR:
@@ -136,6 +139,7 @@ static FAST_CALL void garbage(Obj *o1, int i) {
     case ERROR_ZERO_NEGPOWER:
     case ERROR__NOT_ONE_CHAR:
     case ERROR______NOT_UVAL:
+    case ERROR_CANT_CROSS_BA:
         v = v1->u.obj;
         break;
     case ERROR__INVALID_CONV:
@@ -151,7 +155,7 @@ static FAST_CALL void garbage(Obj *o1, int i) {
     } else v->refcount++;
 }
 
-MALLOC Error *new_error(Error_types num, linepos_t epoint) {
+MUST_CHECK Error *new_error(Error_types num, linepos_t epoint) {
     Error *v = Error(val_alloc(ERROR_OBJ));
     v->num = num;
     v->file_list = current_file_list;
@@ -167,24 +171,24 @@ MALLOC Error *new_error(Error_types num, linepos_t epoint) {
     return v;
 }
 
-MALLOC Obj *new_error_mem(linepos_t epoint) {
+MUST_CHECK Obj *new_error_mem(linepos_t epoint) {
     return Obj(new_error(ERROR_OUT_OF_MEMORY, epoint));
 }
 
-MALLOC Obj *new_error_obj(Error_types num, Obj *v1, linepos_t epoint) {
+MUST_CHECK Obj *new_error_obj(Error_types num, Obj *v1, linepos_t epoint) {
     Error *v = new_error(num, epoint);
     v->u.obj = val_reference(v1);
     return Obj(v);
 }
 
-MALLOC Obj *new_error_conv(Obj *v1, Type *t, linepos_t epoint) {
+MUST_CHECK Obj *new_error_conv(Obj *v1, Type *t, linepos_t epoint) {
     Error *v = new_error(ERROR__INVALID_CONV, epoint);
     v->u.conv.t = t;
     v->u.conv.val = val_reference(v1);
     return Obj(v);
 }
 
-MALLOC Obj *new_error_argnum(argcount_t num, argcount_t min, argcount_t max, linepos_t epoint) {
+MUST_CHECK Obj *new_error_argnum(argcount_t num, argcount_t min, argcount_t max, linepos_t epoint) {
     Error *v = new_error(ERROR__WRONG_ARGNUM, epoint);
     v->u.argnum.num = num;
     v->u.argnum.min = min;
@@ -285,6 +289,7 @@ void error_obj_update(Error *err, const Obj *v1, Obj *v2) {
     case ERROR______NOT_ITER:
     case ERROR___MATH_DOMAIN:
     case ERROR_LOG_NON_POSIT:
+    case ERROR______LOG_BASE:
     case ERROR_SQUARE_ROOT_N:
     case ERROR___INDEX_RANGE:
     case ERROR_____KEY_ERROR:
@@ -292,6 +297,7 @@ void error_obj_update(Error *err, const Obj *v1, Obj *v2) {
     case ERROR_ZERO_NEGPOWER:
     case ERROR__NOT_ONE_CHAR:
     case ERROR______NOT_UVAL:
+    case ERROR_CANT_CROSS_BA:
         if (err->u.obj == v1) {
             val_replace(&err->u.obj, v2);
         }
