@@ -1,5 +1,5 @@
 /*
-    $Id: eval.c 3114 2023-09-06 20:55:15Z soci $
+    $Id: eval.c 3235 2025-05-04 15:37:02Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -94,6 +94,32 @@ FAST_CALL size_t get_label(const uint8_t *s) {
     }
 }
 
+FAST_CALL size_t get_label2(const uint8_t *s, const uint8_t *e) {
+    const uint8_t *i;
+    if (s >= e) return 0;
+    if likely(((uint8_t)((*s | 0x20) - 'a')) <= 'z' - 'a' || *s == '_') {
+        i = s + 1;
+    } else {
+        unsigned int l;
+        if (*s < 0x80) return 0;
+        l = get_label_start(s);
+        if (l == 0) return 0;
+        i = s + l;
+    }
+    while (i < e) {
+        unsigned int l;
+        if likely(((uint8_t)((*i | 0x20) - 'a')) <= 'z' - 'a' || (*i ^ 0x30) < 10 || *i == '_') {
+            i++;
+            continue;
+        }
+        if (*i < 0x80) return (size_t)(i - s);
+        l = get_label_continue(i);
+        if (l == 0) return (size_t)(i - s);
+        i += l;
+    }
+    return (size_t)(i - s);
+}
+
 static MUST_CHECK Obj *get_dec(linepos_t epoint) {
     linecpos_t len, len2;
     Obj *v = int_from_decstr(pline + lpoint.pos, &len, &len2);
@@ -315,7 +341,7 @@ static uval_t bitscalc(address_t addr, Bits *val) {
 
 static uval_t bytescalc(address_t addr, Bytes *val) {
     size_t b = val->len < 0 ? (size_t)~val->len : (size_t)val->len;
-    if (b >= 8 * sizeof(addr)) return (uval_t)b;
+    if (b >= sizeof(addr)) return (uval_t)b;
     if ((addr >> (b << 3)) == 0) return (uval_t)b;
     if (addr <= 0xff) return 1;
     if (addr <= 0xffff) return 2;
@@ -349,7 +375,7 @@ MUST_CHECK Obj *get_star(void) {
     code->apass = pass;
     code->memblocks = ref_memblocks(current_address->mem);
     code->names = ref_namespace(current_context);
-    code->requires = current_section->requires;
+    code->required = current_section->required;
     code->conflicts = current_section->conflicts;
     code->memaddr = current_address->address;
     code->membp = 0;
@@ -868,6 +894,10 @@ static MUST_CHECK Obj *apply_addressing(Obj *o1, atype_t addrtype, bool inplace)
             return val_reference(o1);
         }
         return new_address(val_reference(v1->val), addrtype | (v1->type << 4));
+    } 
+    if (o1->obj == CODE_OBJ && Code(o1)->typ->obj == ADDRESS_OBJ) {
+        atype_t am = Address(Code(o1)->typ)->type;
+        return new_address(code_remove_address(Code(o1), inplace), addrtype | (am << 4));
     }
     return new_address(val_reference(o1), addrtype);
 }
@@ -921,6 +951,7 @@ static bool get_val2(struct eval_context_s *ev) {
                 tmp.val = v + 1;
                 tmp.len = args; /* assumes no referencing */
                 tmp.v.obj = FUNCARGS_OBJ;
+                tmp.v.refcount = 1;
 
                 epoint.pos = out->pos;
                 v--;
@@ -978,7 +1009,7 @@ static bool get_val2(struct eval_context_s *ev) {
                     case 1:
                     ind1:
                         am = (op == O_BRACKET) ? A_LI : A_I;
-                        if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable) {
+                        if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable && (v[1].val->obj != CODE_OBJ || Code(v[1].val)->typ->obj != ADDRESS_OBJ)) {
                             v[0].val = new_address(v[1].val, am);
                         } else {
                             v[0].val = apply_addressing(v[1].val, am, true);
@@ -987,13 +1018,13 @@ static bool get_val2(struct eval_context_s *ev) {
                         v[1].val = NULL;
                         continue;
                     case 2:
-                        if (v[2].val->obj != REGISTER_OBJ || Register(v[2].val)->len != 1) goto nind2;
-                        am = register_to_indexing(Register(v[2].val)->data[0]);
+                        if (v[2].val->obj != REGISTER_OBJ) goto nind2;
+                        am = register_to_indexing(Register(v[2].val));
                         if (am == A_NONE) goto nind2;
                     ind2:
                         val_destroy(v[2].val);
                         v[2].val = NULL;
-                        if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable) {
+                        if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable && (v[1].val->obj != CODE_OBJ || Code(v[1].val)->typ->obj != ADDRESS_OBJ)) {
                             v[1].val = new_address(v[1].val, am);
                         } else {
                             Obj *tmp = apply_addressing(v[1].val, am, true);
@@ -1002,16 +1033,16 @@ static bool get_val2(struct eval_context_s *ev) {
                         }
                         goto ind1;
                     case 3:
-                        if (v[3].val->obj != REGISTER_OBJ || Register(v[3].val)->len != 1) goto nind2;
-                        am2 = register_to_indexing(Register(v[3].val)->data[0]);
+                        if (v[3].val->obj != REGISTER_OBJ) goto nind2;
+                        am2 = register_to_indexing(Register(v[3].val));
                         if (am2 == A_NONE) goto nind2;
-                        if (v[2].val->obj != REGISTER_OBJ || Register(v[2].val)->len != 1) goto nind2;
-                        am = register_to_indexing(Register(v[2].val)->data[0]);
+                        if (v[2].val->obj != REGISTER_OBJ) goto nind2;
+                        am = register_to_indexing(Register(v[2].val));
                         if (am == A_NONE) goto nind2;
                         val_destroy(v[2].val);
                         v[2].val = v[3].val;
                         v[3].val = NULL;
-                        if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable) {
+                        if (v[1].val->obj != ADDRESS_OBJ && !v[1].val->obj->iterable && (v[1].val->obj != CODE_OBJ || Code(v[1].val)->typ->obj != ADDRESS_OBJ)) {
                             v[1].val = new_address(v[1].val, am);
                         } else {
                             Obj *tmp = apply_addressing(v[1].val, am, true);
@@ -1100,6 +1131,7 @@ static bool get_val2(struct eval_context_s *ev) {
                 tmp.val = v;
                 tmp.len = 3; /* assumes no referencing */
                 tmp.v.obj = FUNCARGS_OBJ;
+                tmp.v.refcount = 1;
 
                 epoint.pos = out->pos;
                 oper.v1 = none_value;
@@ -1184,7 +1216,7 @@ static bool get_val2(struct eval_context_s *ev) {
         addr:
             if (vsp == 0) goto syntaxe;
             v = &values[vsp - 1];
-            if (v[0].val->obj != ADDRESS_OBJ && !v[0].val->obj->iterable) {
+            if (v[0].val->obj != ADDRESS_OBJ && !v[0].val->obj->iterable && (v[0].val->obj != CODE_OBJ || Code(v[0].val)->typ->obj != ADDRESS_OBJ)) {
                 v[0].val = new_address(v[0].val, am);
             } else {
                 val = apply_addressing(v[0].val, am, true);
