@@ -1,5 +1,5 @@
 /*
-    $Id: file.c 3112 2023-09-06 06:34:22Z soci $
+    $Id: file.c 3234 2025-05-04 14:29:35Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include "file.h"
 #include <string.h>
 #include <errno.h>
+#include <time.h>
 #ifdef _WIN32
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
@@ -186,7 +187,7 @@ static wchar_t *get_real_name(const wchar_t *name) {
         if (tmp != NULL) {
             short_name = tmp;
             len = ret;
-            ret = GetShortPathNameW(name, real_name, len);
+            ret = GetShortPathNameW(name, short_name, len);
         }
     }
     if (ret >= len || ret == 0) {
@@ -313,9 +314,9 @@ static struct {
     time_t value;
 } latest_file_time;
 
-bool get_latest_file_time(void *time) {
+bool get_latest_file_time(void *ftime) {
     if (!latest_file_time.valid || latest_file_time.current) return true;
-    *((time_t *)time) = latest_file_time.value;
+    *((time_t *)ftime) = latest_file_time.value;
     return false;
 }
 
