@@ -1,5 +1,5 @@
 /*
-    $Id: functionobj.c 3096 2023-09-03 10:24:03Z soci $
+    $Id: functionobj.c 3227 2025-05-01 19:19:17Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -469,35 +469,33 @@ static MUST_CHECK Obj *function_binary(oper_t op) {
 
 static Obj *function_unsigned_bytes(oper_t op, unsigned int bits) {
     uval_t uv;
-    if (touval(op->v2, &uv, bits, op->epoint2)) uv = 0;
-    return bytes_from_uval(uv, bits >> 3);
+    Error *err = op->v2->obj->uval(op->v2, &uv, bits, op->epoint2);
+    return err != NULL ? Obj(err) : bytes_from_uval(uv, bits >> 3);
 }
 
 static Obj *function_signed_bytes(oper_t op, unsigned int bits) {
     ival_t iv;
-    if (toival(op->v2, &iv, bits, op->epoint2)) iv = 0;
-    return bytes_from_uval((uval_t)iv, bits >> 3);
+    Error *err = op->v2->obj->ival(op->v2, &iv, bits, op->epoint2);
+    return err != NULL ? Obj(err) : bytes_from_uval((uval_t)iv, bits >> 3);
 }
 
 static Obj *function_rta_addr(oper_t op, bool rta) {
     uval_t uv;
     Obj *val = op->v2;
     atype_t am = val->obj->address(val);
-    if (touaddress(val, &uv, (am == A_KR) ? 16 : all_mem_bits, op->epoint2)) {
-        uv = 0;
-    } else {
-        uv &= all_mem;
-        switch (am) {
-        case A_NONE:
-            if ((current_address->l_address ^ uv) > 0xffff) err_msg2(ERROR_CANT_CROSS_BA, val, op->epoint2);
-            break;
-        case A_KR:
-            break;
-        default:
-            err_msg_output_and_destroy(err_addressing(am, op->epoint2, -1));
-        }
-        if (rta) uv--;
+    Error *err = val->obj->uaddress(val, &uv, (am == A_KR) ? 16 : all_mem_bits, op->epoint2);
+    if (err != NULL) return Obj(err);
+    uv &= all_mem;
+    switch (am) {
+    case A_NONE:
+        if ((current_address->l_address ^ uv) > 0xffff) return new_error_obj(ERROR_CANT_CROSS_BA, val, op->epoint2);
+        break;
+    case A_KR:
+        break;
+    default:
+        return Obj(err_addressing(am, op->epoint2, -1));
     }
+    if (rta) uv--;
     return bytes_from_uval(uv, 2);
 }
 
@@ -551,12 +549,6 @@ static MUST_CHECK Obj *function_function(oper_t op) {
         }
         real = log10(real);
         break;
-    case F_LOG:
-        if (real <= 0.0) {
-            err = ERROR_LOG_NON_POSIT; goto failed;
-        }
-        real = log(real);
-        break;
     case F_EXP: real = exp(real);break;
     case F_SIN: real = sin(real);break;
     case F_COS: real = cos(real);break;
@@ -651,13 +643,36 @@ static MUST_CHECK Obj *to_real(struct values_s *v, double *r) {
 static MUST_CHECK Obj *function_hypot(oper_t op) {
     struct values_s *v = Funcargs(op->v2)->val;
     Obj *val;
-    double real, real2;
+    double real, result;
+    argcount_t i, len = Funcargs(op->v2)->len;
 
-    val = to_real(&v[0], &real);
-    if (val != NULL) return val;
-    val = to_real(&v[1], &real2);
-    if (val != NULL) return val;
-    return float_from_double(hypot(real, real2), op->epoint);
+    if (len == 0) {
+        result = 0.0;
+    } else {
+        val = to_real(&v[0], &result);
+        if (val != NULL) return val;
+
+        switch (len) {
+        case 1:
+            if (result < 0) result = -result;
+            break;
+        case 2:
+            val = to_real(&v[1], &real);
+            if (val != NULL) return val;
+            result = hypot(result, real);
+            break;
+        default:
+            result *= result;
+            for (i = 1; i < len; i++) {
+                val = to_real(&v[i], &real);
+                if (val != NULL) return val;
+                result += real * real;
+            }
+            result = sqrt(result);
+            break;
+        }
+    }
+    return float_from_double(result, op->epoint);
 }
 
 static MUST_CHECK Obj *function_atan2(oper_t op) {
@@ -673,21 +688,46 @@ static MUST_CHECK Obj *function_atan2(oper_t op) {
 }
 
 static MUST_CHECK Obj *function_pow(oper_t op) {
+    struct values_s *v = Funcargs(op->v2)->val;
+    op->op = O_EXP;
+    op->epoint3 = op->epoint;
+    op->epoint = &v[0].epoint;
+    op->epoint2 = &v[1].epoint;
+    op->v1 = v[0].val;
+    op->v2 = v[1].val;
+    return op->v1->obj->calc2(op);
+}
+
+static MUST_CHECK Obj *function_log(oper_t op) {
     struct values_s *v = Funcargs(op->v2)->val;
     Obj *val;
     double real, real2;
 
     val = to_real(&v[0], &real);
     if (val != NULL) return val;
-    val = to_real(&v[1], &real2);
-    if (val != NULL) return val;
-    if (real2 < 0.0 && real == 0.0) {
-        return new_error_obj(ERROR_ZERO_NEGPOWER, op->v2, op->epoint2);
-    }
-    if (real < 0.0 && floor(real2) != real2) {
-        return Obj(new_error(ERROR_NEGFRAC_POWER, op->epoint2));
-    }
-    return float_from_double(pow(real, real2), op->epoint);
+    if (real <= 0.0) return new_error_obj(ERROR_LOG_NON_POSIT, v[0].val, &v[0].epoint);
+
+    if (Funcargs(op->v2)->len == 2) {
+        val = to_real(&v[1], &real2);
+        if (val != NULL) return val;
+        if (real2 == 10.0) real = log10(real);
+        else if (real2 == 2.0) {
+            int e;
+            real = frexp(real, &e);
+            if (real < 0.707106781186547524400844362104849039284) {
+                real *= 2.0;
+                e--;
+            }
+            real = (double)e + log(real)*1.44269504088896340735992468100189213743;
+        } else {
+            if (real2 > 0.0) real2 = log(real2); else real2 = 0.0;
+            if (real2 == 0.0 || real2 == HUGE_VAL || real2 == -HUGE_VAL || real2 != real2) {
+                return new_error_obj(ERROR______LOG_BASE, v[1].val, &v[1].epoint);
+            }
+            real = log(real) / real2;
+        }
+    } else real = log(real);
+    return float_from_double(real, op->epoint);
 }
 
 static MUST_CHECK Obj *function_all_any(oper_t op, Obj **warn, bool first) {
@@ -772,9 +812,6 @@ static MUST_CHECK Obj *calc2(oper_t op) {
                 func = v1->func;
                 switch (func) {
                 case F_HYPOT:
-                    if (args != 2) {
-                        return new_error_argnum(args, 2, 2, op->epoint3);
-                    }
                     return gen_broadcast(op, function_hypot);
                 case F_ATAN2:
                     if (args != 2) {
@@ -785,7 +822,12 @@ static MUST_CHECK Obj *calc2(oper_t op) {
                     if (args != 2) {
                         return new_error_argnum(args, 2, 2, op->epoint3);
                     }
-                    return gen_broadcast(op, function_pow);
+                    return function_pow(op);
+                case F_LOG:
+                    if (args < 1 || args > 2) {
+                        return new_error_argnum(args, 1, 2, op->epoint3);
+                    }
+                    return gen_broadcast(op, function_log);
                 case F_RANGE:
                     if (args < 1 || args > 3) {
                         return new_error_argnum(args, 1, 3, op->epoint3);
@@ -829,7 +871,7 @@ static MUST_CHECK Obj *calc2(oper_t op) {
                         return function_sort(v->val, &v->epoint);
                     default:
                         op->v2 = v->val;
-                        op->inplace = v->val->refcount == 1 ? v->val : NULL;
+                        op->inplace = o2->refcount == 1 && v->val->refcount == 1 ? v->val : NULL;
                         return apply_func(op, function_function);
                     }
                 }
