@@ -1,5 +1,5 @@
 /*
-    $Id: gapobj.c 2122 2019-12-21 06:27:50Z soci $
+    $Id: gapobj.c 2896 2022-11-05 05:33:41Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,11 +23,8 @@
 #include "values.h"
 
 #include "strobj.h"
-#include "operobj.h"
-#include "intobj.h"
 #include "boolobj.h"
 #include "typeobj.h"
-#include "errorobj.h"
 
 static Type obj;
 
@@ -35,23 +32,13 @@ Type *const GAP_OBJ = &obj;
 
 static Gap gapval = { { &obj, 1 }, NULL};
 
-Gap *gap_value = &gapval;
-
-static MUST_CHECK Obj *create(Obj *v1, linepos_t epoint) {
-    switch (v1->obj->type) {
-    case T_NONE:
-    case T_ERROR:
-    case T_GAP: return val_reference(v1);
-    default: break;
-    }
-    return (Obj *)new_error_conv(v1, GAP_OBJ, epoint);
-}
+Obj *const gap_value = &gapval.v;
 
 static FAST_CALL bool same(const Obj *o1, const Obj *o2) {
     return o1 == o2;
 }
 
-static MUST_CHECK Error *hash(Obj *UNUSED(v1), int *hs, linepos_t UNUSED(epoint)) {
+static MUST_CHECK Obj *hash(Obj *UNUSED(v1), int *hs, linepos_t UNUSED(epoint)) {
     *hs = 0; /* whatever, there's only one */
     return NULL;
 }
@@ -67,7 +54,7 @@ static MUST_CHECK Obj *repr(Obj *UNUSED(v1), linepos_t UNUSED(epoint), size_t ma
         v->data[0] = '?';
         gapval.repr = v;
     }
-    return val_reference(&v->v);
+    return val_reference(Obj(v));
 }
 
 static MUST_CHECK Obj *function(oper_t op) {
@@ -75,7 +62,7 @@ static MUST_CHECK Obj *function(oper_t op) {
 }
 
 static MUST_CHECK Obj *calc1(oper_t op) {
-    switch (op->op->op) {
+    switch (op->op) {
     case O_BANK:
     case O_HIGHER:
     case O_LOWER:
@@ -95,16 +82,7 @@ static MUST_CHECK Obj *calc2(oper_t op) {
     Obj *v2 = op->v2;
     switch (v2->obj->type) {
     case T_GAP:
-        switch (op->op->op) {
-        case O_CMP: return (Obj *)ref_int(int_value[0]);
-        case O_GE:
-        case O_LE:
-        case O_EQ: return (Obj *)ref_bool(true_value);
-        case O_NE:
-        case O_MIN:
-        case O_LT:
-        case O_MAX:
-        case O_GT: return (Obj *)ref_bool(false_value);
+        switch (op->op) {
         case O_ADD:
         case O_SUB:
         case O_MUL:
@@ -118,7 +96,7 @@ static MUST_CHECK Obj *calc2(oper_t op) {
         case O_RSHIFT: return val_reference(op->v1);
         default: break;
         }
-        break;
+        return obj_oper_compare(op, 0);
     case T_STR:
     case T_BOOL:
     case T_INT:
@@ -128,9 +106,9 @@ static MUST_CHECK Obj *calc2(oper_t op) {
     case T_ADDRESS:
     case T_BYTES:
     case T_REGISTER:
-        switch (op->op->op) {
-        case O_EQ: return (Obj *)ref_bool(false_value);
-        case O_NE: return (Obj *)ref_bool(true_value);
+        switch (op->op) {
+        case O_EQ: return ref_false();
+        case O_NE: return ref_true();
         case O_ADD:
         case O_SUB:
         case O_MUL:
@@ -149,7 +127,7 @@ static MUST_CHECK Obj *calc2(oper_t op) {
     case T_ERROR:
         return val_reference(v2);
     default:
-        if (v2->obj->iterable && op->op != &o_MEMBER && op->op != &o_X) {
+        if (v2->obj->iterable && op->op != O_MEMBER && op->op != O_X) {
             return v2->obj->rcalc2(op);
         }
         break;
@@ -170,9 +148,9 @@ static MUST_CHECK Obj *rcalc2(oper_t op) {
     case T_ADDRESS:
     case T_BYTES:
     case T_REGISTER:
-        switch (op->op->op) {
-        case O_EQ: return (Obj *)ref_bool(false_value);
-        case O_NE: return (Obj *)ref_bool(true_value);
+        switch (op->op) {
+        case O_EQ: return ref_false();
+        case O_NE: return ref_true();
         case O_ADD:
         case O_SUB:
         case O_MUL:
@@ -185,40 +163,39 @@ static MUST_CHECK Obj *rcalc2(oper_t op) {
         default: break;
         }
         break;
+    default:
+        if (!v1->obj->iterable) {
+            break;
+        }
+        FALL_THROUGH; /* fall through */
     case T_NONE:
     case T_ERROR:
-    case T_TUPLE:
-    case T_LIST:
-        if (op->op != &o_IN) {
-            return v1->obj->calc2(op);
-        }
-        break;
-    default: break;
+        return v1->obj->calc2(op);
     }
     return obj_oper_error(op);
 }
 
 void gapobj_init(void) {
-    new_type(&obj, T_GAP, "gap", sizeof(Gap));
-    obj.create = create;
-    obj.same = same;
-    obj.hash = hash;
-    obj.repr = repr;
-    obj.function = function;
-    obj.calc1 = calc1;
-    obj.calc2 = calc2;
-    obj.rcalc2 = rcalc2;
+    Type *type = new_type(&obj, T_GAP, "gap", sizeof(Gap));
+    type->same = same;
+    type->hash = hash;
+    type->repr = repr;
+    type->function = function;
+    type->calc1 = calc1;
+    type->calc2 = calc2;
+    type->rcalc2 = rcalc2;
+    gapval.repr = NULL;
 }
 
 void gapobj_names(void) {
-    new_builtin("gap", val_reference(&GAP_OBJ->v));
+    new_builtin("gap", val_reference(Obj(GAP_OBJ)));
 }
 
 void gapobj_destroy(void) {
 #ifdef DEBUG
-    if (gap_value->v.refcount != 1) fprintf(stderr, "gap %" PRIuSIZE "\n", gap_value->v.refcount - 1);
+    if (gap_value->refcount != 1) fprintf(stderr, "gap %" PRIuSIZE "\n", gap_value->refcount - 1);
     if (gapval.repr != NULL && gapval.repr->v.refcount != 1) fprintf(stderr, "gaprepr %" PRIuSIZE "\n", gapval.repr->v.refcount - 1);
 #endif
 
-    if (gapval.repr != NULL) val_destroy(&gapval.repr->v);
+    if (gapval.repr != NULL) val_destroy(Obj(gapval.repr));
 }
