@@ -1,5 +1,5 @@
 /*
-    $Id: intobj.c 3086 2023-09-03 06:23:08Z soci $
+    $Id: intobj.c 3229 2025-05-04 10:20:30Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -92,7 +92,7 @@ static FAST_CALL void destroy(Obj *o1) {
     if unlikely(v1->val != v1->data) int_destroy(v1);
 }
 
-static inline MALLOC Int *new_int(void) {
+static inline MUST_CHECK Int *new_int(void) {
     return Int(val_alloc(INT_OBJ));
 }
 
@@ -775,27 +775,42 @@ static MUST_CHECK Obj *idivrem(oper_t op, bool divrem) {
 }
 
 static inline MUST_CHECK Obj *power(oper_t op) {
-    const Int *vv1 = Int(op->v1), *vv2 = Int(op->v2);
-    int j;
-    bool neg = false;
-    size_t i;
-    Int *v = new_int();
-    v->data = v->val;
-    v->val[0] = 1;
-    v->len = 1;
+    Int *vv1 = Int(op->v1), *vv2 = Int(op->v2);
+    digit_t d, j;
+    size_t i, ln;
+    Int *v;
 
-    for (i = (size_t)vv2->len; (i--) != 0;) {
-        digit_t d = vv2->data[i];
-        for (j = SHIFT - 1; j >= 0; j--) {
-            imul(v, v, v);
-            if ((d & (1U << j)) != 0) {
-                imul(v, vv1, v);
-                neg = true;
-            } else neg = false;
+    i = (size_t)vv2->len;
+    while (i != 0) {
+        d = vv2->data[--i];
+        for (j = (digit_t)1 << (SHIFT - 1); j != 0; j >>= 1) {
+            if ((d & j) != 0) goto found;
         }
     }
-    if (neg && vv1->len < 0) v->len = -v->len;
+    return val_reference(int_value[1]);
+found:
+    v = new_int();
+    ln = intlen(vv1);
+    v->len = (ssize_t)ln;
+    v->data = inew2(v, ln);
+    if (v->data == NULL) goto failed2;
+    memcpy(v->data, vv1->data, ln * sizeof *v->data);
+    for (j >>= 1; j != 0; j >>= 1) {
+        imul(v, v, v);
+        if ((d & j) != 0) imul(v, vv1, v);
+    }
+    while (i != 0) {
+        d = vv2->data[--i];
+        for (j = (digit_t)1 << (SHIFT - 1); j != 0; j >>= 1) {
+            imul(v, v, v);
+            if ((d & j) != 0) imul(v, vv1, v);
+        }
+    }
+    if (vv1->len < 0 && (vv2->data[0] & 1) != 0) v->len = -v->len;
     return Obj(v);
+failed2:
+    val_destroy(Obj(v));
+    return new_error_mem(op->epoint3);
 }
 
 static MUST_CHECK Obj *lshift(oper_t op, uval_t s) {
@@ -1728,6 +1743,7 @@ static MUST_CHECK Obj *calc2_int(oper_t op) {
             val_destroy(vv2);
             return val;
         }
+        if (v1->len == 0) return val_reference(Obj(v1));
         return power(op);
     case O_LSHIFT:
         err = ival(Obj(v2), &shift, 8 * sizeof shift, op->epoint2);
