@@ -1,5 +1,5 @@
 /*
-    $Id: labelobj.c 2083 2019-11-16 05:49:12Z soci $
+    $Id: labelobj.c 2675 2021-05-20 20:53:26Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,32 +25,21 @@
 
 #include "strobj.h"
 #include "typeobj.h"
-#include "errorobj.h"
 
 static Type obj;
 
 Type *const LABEL_OBJ = &obj;
 
-static MUST_CHECK Obj *create(Obj *v1, linepos_t epoint) {
-    switch (v1->obj->type) {
-    case T_NONE:
-    case T_ERROR:
-    case T_LABEL: return val_reference(v1);
-    default: break;
-    }
-    return (Obj *)new_error_conv(v1, LABEL_OBJ, epoint);
-}
-
 static FAST_CALL void destroy(Obj *o1) {
-    Label *v1 = (Label *)o1;
+    Label *v1 = Label(o1);
     const struct file_s *cfile = v1->file_list->file;
-    if ((size_t)(v1->name.data - cfile->data) >= cfile->len) free((char *)v1->name.data);
+    if (not_in_file(v1->name.data, cfile)) free((char *)v1->name.data);
     if (v1->name.data != v1->cfname.data) free((uint8_t *)v1->cfname.data);
     val_destroy(v1->value);
 }
 
 static FAST_CALL void garbage(Obj *o1, int i) {
-    Label *v1 = (Label *)o1;
+    Label *v1 = Label(o1);
     Obj *v;
     const struct file_s *cfile;
     switch (i) {
@@ -59,7 +48,7 @@ static FAST_CALL void garbage(Obj *o1, int i) {
         return;
     case 0:
         cfile = v1->file_list->file;
-        if ((size_t)(v1->name.data - cfile->data) >= cfile->len) free((char *)v1->name.data);
+        if (not_in_file(v1->name.data, cfile)) free((char *)v1->name.data);
         if (v1->name.data != v1->cfname.data) free((uint8_t *)v1->cfname.data);
         return;
     case 1:
@@ -73,8 +62,8 @@ static FAST_CALL void garbage(Obj *o1, int i) {
 }
 
 static FAST_CALL bool same(const Obj *o1, const Obj *o2) {
-    const Label *v1 = (const Label *)o1, *v2 = (const Label *)o2;
-    return o2->obj == LABEL_OBJ
+    const Label *v1 = Label(o1), *v2 = Label(o2);
+    return o1->obj == o2->obj
         && v1->epoint.pos == v2->epoint.pos
         && v1->epoint.line == v2->epoint.line
         && v1->file_list == v2->file_list
@@ -84,7 +73,7 @@ static FAST_CALL bool same(const Obj *o1, const Obj *o2) {
 }
 
 static MUST_CHECK Obj *repr(Obj *o1, linepos_t epoint, size_t maxlen) {
-    Label *v1 = (Label *)o1;
+    Label *v1 = Label(o1);
     size_t len, len2;
     uint8_t *s;
     Str *v;
@@ -107,11 +96,11 @@ static MUST_CHECK Obj *repr(Obj *o1, linepos_t epoint, size_t maxlen) {
     memcpy(s, v1->name.data, len);
     s[len] = '\'';
     s[len + 1] = '>';
-    return &v->v;
+    return Obj(v);
 }
 
 static MUST_CHECK Obj *str(Obj *o1, linepos_t UNUSED(epoint), size_t maxlen) {
-    Label *v1 = (Label *)o1;
+    Label *v1 = Label(o1);
     size_t len, chars;
     Str *v;
     switch (v1->name.data[0]) {
@@ -127,15 +116,14 @@ static MUST_CHECK Obj *str(Obj *o1, linepos_t UNUSED(epoint), size_t maxlen) {
     if (v == NULL) return NULL;
     v->chars = chars;
     memcpy(v->data, v1->name.data, len);
-    return &v->v;
+    return Obj(v);
 }
 
 void labelobj_init(void) {
-    new_type(&obj, T_LABEL, "label", sizeof(Label));
-    obj.create = create;
-    obj.destroy = destroy;
-    obj.garbage = garbage;
-    obj.same = same;
-    obj.repr = repr;
-    obj.str = str;
+    Type *type = new_type(&obj, T_LABEL, "label", sizeof(Label));
+    type->destroy = destroy;
+    type->garbage = garbage;
+    type->same = same;
+    type->repr = repr;
+    type->str = str;
 }
