@@ -1,5 +1,5 @@
 /*
-    $Id: listing.c 3112 2023-09-06 06:34:22Z soci $
+    $Id: listing.c 3191 2025-04-08 16:42:51Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -137,18 +137,20 @@ static void out_txt(Listing *ls, const char *s) {
 
 static Listing *listing;
 
-void listing_open(const struct list_output_s *output, int argc, char *argv[]) {
+bool listing_open(const struct list_output_s *output, int argc, char *argv[]) {
     static Listing listing2;
     Listing *ls;
     time_t t;
     int i;
     FILE *flist;
 
+    if (output->name == NULL) return false;
+
     flist = dash_name(output->name) ? stdout : fopen_utf8(output->name, output->append ? "at" : "wt");
     if (flist == NULL) {
         err_msg_file2(ERROR_CANT_WRTE_LST, output->name, &output->name_pos);
         listing = NULL;
-        return;
+        return false;
     }
     if (flist == stdout && fflush(flist) != 0) setvbuf(flist, NULL, _IOLBF, 1024);
     clearerr(flist); errno = 0;
@@ -203,6 +205,7 @@ void listing_open(const struct list_output_s *output, int argc, char *argv[]) {
     flushbuf(ls);
     newline(ls);
     listing = ls;
+    return true;
 }
 
 void listing_close(const struct list_output_s *output) {
@@ -330,6 +333,7 @@ static void printmon(Listing *ls, unsigned int cod, int ln, uint32_t adr) {
     mode++;
     switch (type) {
     case ADR_IMPLIED: ls->s--; return;
+    case ADR_LEN:
     case ADR_REG: return;
     case ADR_IMMEDIATE:
         switch (ln) {
@@ -338,11 +342,12 @@ static void printmon(Listing *ls, unsigned int cod, int ln, uint32_t adr) {
         case 1: out_byte(ls, adr); return;
         case 2: out_word(ls, adr); return;
         }
-    case ADR_ADDR: if (cod == 0x20 || cod == 0x4c) out_pb(ls, adr); else out_db(ls, adr); return;
+    case ADR_ADDR: out_db(ls, adr); return;
+    case ADR_ADDR_K: out_pb(ls, adr); return;
     case ADR_BIT_ZP: out_bit(ls, cod, adr); return;
     case ADR_LONG:
     case ADR_LONG_X: out_long(ls, adr); break;
-    case ADR_ADDR_X_I: out_pb(ls, adr); break;
+    case ADR_ADDR_K_X_I: out_pb(ls, adr); break;
     case ADR_ZP_R:
     case ADR_ZP_R_I_Y:
     case ADR_ZP_S:
@@ -355,11 +360,12 @@ static void printmon(Listing *ls, unsigned int cod, int ln, uint32_t adr) {
     case ADR_ZP_I_Z:
     case ADR_ZP_LI:
     case ADR_ZP_LI_Y:
+    case ADR_ZP_LI_Z:
     case ADR_ZP_X:
     case ADR_ZP_X_I:
     case ADR_ZP_Y: out_zp(ls, adr); break;
-    case ADR_ADDR_I:
-    case ADR_ADDR_LI: out_word(ls, adr); break;
+    case ADR_ADDR_0_I:
+    case ADR_ADDR_0_LI: out_word(ls, adr); break;
     case ADR_REL: if (ln > 0) out_pb(ls, (address_t)((int8_t)adr + (int)current_address->l_address)); else ls->s--; return;
     case ADR_BIT_ZP_REL:
         out_bit(ls, cod, adr);
