@@ -1,5 +1,5 @@
 /*
-    $Id: macro.c 3086 2023-09-03 06:23:08Z soci $
+    $Id: macro.c 3221 2025-04-21 19:48:07Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -212,7 +212,7 @@ bool mtranslate(void) {
                     if (macro->param[j].cfname.len != cf.len) continue;
                     data = macro->param[j].cfname.data;
                     if (data[0] != cf.data[0]) continue;
-                    if (memcmp(data, cf.data, cf.len) == 0) break;
+                    if (cf.len == 1 || memcmp(data, cf.data, cf.len) == 0) break;
                 }
                 if (j < macro->argc) break;
                 lpoint.pos = (linecpos_t)(op - pline);
@@ -521,6 +521,7 @@ Obj *mfunc_recurse(Mfunc *mfunc, Namespace *context, uint8_t strength, linepos_t
                         tmp.val = vs;
                         tmp.len = 1; /* assumes no referencing */
                         tmp.v.obj = FUNCARGS_OBJ;
+                        tmp.v.refcount = 1;
                         oper.v2 = &tmp.v;
                         val = oper.v1->obj->calc2(&oper);
                     }
@@ -529,13 +530,14 @@ Obj *mfunc_recurse(Mfunc *mfunc, Namespace *context, uint8_t strength, linepos_t
         }
         label = new_label(&param->name, context, strength, mfunc->file_list);
         if (label->value != NULL) {
-            if (label->constant) {
+            if (label->defpass == pass && label->constant) {
                 err_msg_double_defined(label, &param->name, &param->epoint); /* not possible in theory */
                 val_destroy(val);
             } else {
                 if (label->defpass != pass) {
                     label->ref = false;
                     label->defpass = pass;
+                    label->constant = false;
                 } else {
                     if (diagnostics.unused.variable && label->usepass != pass) err_msg_unused_variable(label);
                 }
@@ -625,7 +627,6 @@ bool get_func_params(Mfunc *v, bool single) {
             if (single) {
                 const uint8_t *s = pline + lpoint.pos;
                 if (!stard && s[0] != ',' && s[0] != ':' && (s[0] != '=' || s[1] == '=')) {
-                    v->epoint.line = lpoint.line - 1;
                     lpoint.pos = param->epoint.pos;
                     v->epoint.pos = lpoint.pos;
                     break;
@@ -659,7 +660,6 @@ bool get_func_params(Mfunc *v, bool single) {
                     }
                     lpoint.pos++;
                     ignore();
-                    v->epoint.line = lpoint.line - 1;
                     v->epoint.pos = lpoint.pos;
                 }
                 break;
@@ -895,6 +895,7 @@ Obj *mfunc2_recurse(Mfunc *mfunc, Funcargs *v2, linepos_t epoint) {
                         tmp.val = &vs;
                         tmp.len = 1; /* assumes no referencing */
                         tmp.v.obj = FUNCARGS_OBJ;
+                        tmp.v.refcount = 1;
                         oper.v2 = &tmp.v;
                         functionrecursion++;
                         val = oper.v1->obj->calc2(&oper);
@@ -905,13 +906,14 @@ Obj *mfunc2_recurse(Mfunc *mfunc, Funcargs *v2, linepos_t epoint) {
         }
         label = new_label(&param->name, context, 0, mfunc->file_list);
         if (label->value != NULL) {
-            if (label->constant) {
+            if (label->defpass == pass && label->constant) {
                 err_msg_double_defined(label, &param->name, &param->epoint);
                 val_destroy(val);
             } else {
                 if (label->defpass != pass) {
                     label->ref = false;
                     label->defpass = pass;
+                    label->constant = false;
                 } else {
                     if (diagnostics.unused.variable && label->usepass != pass) err_msg_unused_variable(label);
                 }
@@ -964,6 +966,7 @@ Obj *mfunc2_recurse(Mfunc *mfunc, Funcargs *v2, linepos_t epoint) {
         push_context(context);
         functionrecursion++;
         if (mfunc->v.obj == SFUNC_OBJ) {
+            lpoint.line--;
             if (mtranslate()) {
                 lpoint.pos = mfunc->epoint.pos;
                 if (mfunc->line != NULL) pline = mfunc->line;
