@@ -1,5 +1,5 @@
 /*
-    $Id: macroobj.h 2016 2019-10-20 06:41:22Z soci $
+    $Id: macroobj.h 2651 2021-05-09 19:33:48Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,21 +34,24 @@ struct macro_param_s {
 
 typedef struct Macro {
     Obj v;
-    size_t argc;
+    argcount_t argc;
+    linenum_t line;
     struct macro_param_s *param;
     const struct file_list_s *file_list;
-    line_t line;
     bool retval;
     uint8_t recursion_pass;
 } Macro;
 typedef struct Macro Segment;
 
+#define Macro(a) OBJ_CAST(Macro, a)
+#define Segment(a) OBJ_CAST(Segment, a)
+
 typedef struct Struct {
     Obj v;
-    size_t argc;
+    argcount_t argc;
+    linenum_t line;
     struct macro_param_s *param;
     const struct file_list_s *file_list;
-    line_t line;
     bool retval;
     uint8_t recursion_pass;
     address_t size; /* first part same as macro! */
@@ -56,6 +59,9 @@ typedef struct Struct {
 } Struct;
 typedef struct Struct Union;
 
+#define Struct(a) OBJ_CAST(Struct, a)
+#define Union(a) OBJ_CAST(Union, a)
+
 extern void macroobj_init(void);
 
 #endif
