@@ -1,5 +1,5 @@
 /*
-    $Id: mfuncobj.h 2137 2020-01-02 00:52:17Z soci $
+    $Id: mfuncobj.h 2696 2021-09-12 20:35:03Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,27 +23,34 @@
 #include "stdbool.h"
 
 extern struct Type *const MFUNC_OBJ;
+extern struct Type *const SFUNC_OBJ;
 
 struct mfunc_param_s {
     str_t name;
     str_t cfname;
     Obj *init;
+    Obj *type;
     struct linepos_s epoint;
 };
 
 typedef struct Mfunc {
     Obj v;
-    size_t argc;
+    argcount_t argc;
     struct mfunc_param_s *param;
     const struct file_list_s *file_list;
-    line_t line;
+    struct linepos_s epoint;
     bool retval;
     uint8_t recursion_pass;
     size_t nslen;
     struct Namespace **namespaces, *names;
     size_t ipoint;
     struct List *inamespaces;
+    const uint8_t *line;
 } Mfunc;
+typedef struct Mfunc Sfunc;
+
+#define Mfunc(a) OBJ_CAST(Mfunc, a)
+#define Sfunc(a) OBJ_CAST(Sfunc, a)
 
 extern void mfuncobj_init(void);
 #endif
