@@ -1,5 +1,5 @@
 /*
-    $Id: noneobj.c 2122 2019-12-21 06:27:50Z soci $
+    $Id: noneobj.c 2690 2021-09-08 09:56:34Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,9 +16,11 @@
     51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 */
+#ifdef DEBUG
+#include <stdio.h>
+#endif
 #include "noneobj.h"
 #include "eval.h"
-#include "values.h"
 
 #include "typeobj.h"
 #include "errorobj.h"
@@ -27,20 +29,20 @@ static Type obj;
 
 Type *const NONE_OBJ = &obj;
 
-static None noneval = { { &obj, 1}, NULL };
+static None noneval = { { &obj, 1} };
 
-None *none_value = &noneval;
+Obj *const none_value = &noneval.v;
 
 static FAST_CALL bool same(const Obj *o1, const Obj *o2) {
     return o1 == o2;
 }
 
 static MUST_CHECK Obj *truth(Obj *UNUSED(v1), Truth_types UNUSED(type), linepos_t epoint) {
-    return (Obj *)new_error(ERROR____STILL_NONE, epoint);
+    return Obj(new_error(ERROR____STILL_NONE, epoint));
 }
 
-static MUST_CHECK Error *hash(Obj *UNUSED(v1), int *UNUSED(v), linepos_t epoint) {
-    return new_error(ERROR____STILL_NONE, epoint);
+static MUST_CHECK Obj *hash(Obj *UNUSED(v1), int *UNUSED(v), linepos_t epoint) {
+    return Obj(new_error(ERROR____STILL_NONE, epoint));
 }
 
 static MUST_CHECK Obj *repr(Obj *v1, linepos_t epoint, size_t UNUSED(maxsize)) {
@@ -63,7 +65,7 @@ static MUST_CHECK Obj *rcalc2(oper_t op) {
     return val_reference(op->v1->obj->type == T_ERROR ? op->v1 : op->v2);
 }
 
-static MUST_CHECK Obj *slice(oper_t op, size_t UNUSED(indx)) {
+static MUST_CHECK Obj *slice(oper_t op, argcount_t UNUSED(indx)) {
     return val_reference(op->v1);
 }
 
@@ -92,29 +94,30 @@ static MUST_CHECK Obj *size(oper_t op) {
 }
 
 void noneobj_init(void) {
-    new_type(&obj, T_NONE, "none", sizeof(None));
-    obj.same = same;
-    obj.truth = truth;
-    obj.repr = repr;
-    obj.str = str;
-    obj.hash = hash;
-    obj.calc1 = calc1;
-    obj.calc2 = calc2;
-    obj.rcalc2 = rcalc2;
-    obj.slice = slice;
-    obj.ival = ival;
-    obj.uval = uval;
-    obj.uval2 = uval;
-    obj.iaddress = ival;
-    obj.uaddress = uval;
-    obj.sign = sign;
-    obj.function = function;
-    obj.len = len;
-    obj.size = size;
+    Type *type = new_type(&obj, T_NONE, "none", sizeof(None));
+    type->same = same;
+    type->truth = truth;
+    type->repr = repr;
+    type->str = str;
+    type->hash = hash;
+    type->calc1 = calc1;
+    type->calc2 = calc2;
+    type->rcalc2 = rcalc2;
+    type->slice = slice;
+    type->contains = rcalc2;
+    type->ival = ival;
+    type->uval = uval;
+    type->uval2 = uval;
+    type->iaddress = ival;
+    type->uaddress = uval;
+    type->sign = sign;
+    type->function = function;
+    type->len = len;
+    type->size = size;
 }
 
 void noneobj_destroy(void) {
 #ifdef DEBUG
-    if (none_value->v.refcount != 1) fprintf(stderr, "none %" PRIuSIZE "\n", none_value->v.refcount - 1);
+    if (none_value->refcount != 1) fprintf(stderr, "none %" PRIuSIZE "\n", none_value->refcount - 1);
 #endif
 }
