@@ -1,5 +1,5 @@
 /*
-    $Id: wait_e.h 2886 2022-11-01 08:01:49Z soci $
+    $Id: wait_e.h 2999 2023-08-12 21:46:25Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,6 @@ typedef enum Wait_types {
     W_ENDREPT, W_ENDREPT2, W_ENDREPT3, W_ENDWHILE, W_ENDWHILE2, W_ENDWHILE3,
     W_SEND, W_SEND2, W_PEND, W_FI, W_FI2, W_ENDF, W_ENDF2, W_ENDF3, W_SWITCH,
     W_SWITCH2, W_WEAK, W_WEAK2, W_ENDN, W_ENDN2, W_ENDV, W_ENDV2, W_ENDWITH,
-    W_ENDWITH2, W_ENDENCODE, W_ENDENCODE2
+    W_ENDWITH2, W_ENDENCODE, W_ENDENCODE2, W_ENDALIGNBLK, W_ENDALIGNBLK2
 } Wait_types;
 #endif
