@@ -1,5 +1,5 @@
 /*
-    $Id: wchar.c 3086 2023-09-03 06:23:08Z soci $
+    $Id: wchar.c 3193 2025-04-08 16:57:14Z soci $
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -712,7 +712,7 @@ int wcwidth_v13(unichar_t ch) {
     if (ch == 0xE0001 || (ch >= 0xE0020 && 0xE007F >= ch) || (ch >= 0xE0100 && 0xE01EF >= ch)) {
         return 0;
     }
-    if ((ch >= 0x20000 && 0x2FFFD >= ch) || (0x30000 >= ch && 0x3FFFD >= ch)) {
+    if ((ch >= 0x20000 && 0x2FFFD >= ch) || (ch >= 0x30000 && 0x3FFFD >= ch)) {
         return 2;
     }
     return 1;
